@@ -21,6 +21,7 @@
 */
 
 #include "unicode/utypes.h"
+#include "unicode/char16ptr.h"
 #include "unicode/uobject.h"
 #include "unicode/uset.h"
 #include "unicode/uniset.h"
@@ -87,14 +88,14 @@ uset_addRange(USet* set, UChar32 start, UChar32 end) {
 }
 
 U_CAPI void U_EXPORT2
-uset_addString(USet* set, const UChar* str, int32_t strLen) {
+uset_addString(USet* set, const char16_t* str, int32_t strLen) {
     // UnicodeString handles -1 for strLen
     UnicodeString s(strLen<0, str, strLen);
     ((UnicodeSet*) set)->UnicodeSet::add(s);
 }
 
 U_CAPI void U_EXPORT2
-uset_addAllCodePoints(USet* set, const UChar *str, int32_t strLen) {
+uset_addAllCodePoints(USet* set, const char16_t *str, int32_t strLen) {
     // UnicodeString handles -1 for strLen
     UnicodeString s(str, strLen);
     ((UnicodeSet*) set)->UnicodeSet::addAll(s);
@@ -111,13 +112,13 @@ uset_removeRange(USet* set, UChar32 start, UChar32 end) {
 }
 
 U_CAPI void U_EXPORT2
-uset_removeString(USet* set, const UChar* str, int32_t strLen) {
+uset_removeString(USet* set, const char16_t* str, int32_t strLen) {
     UnicodeString s(strLen==-1, str, strLen);
     ((UnicodeSet*) set)->UnicodeSet::remove(s);
 }
 
 U_CAPI void U_EXPORT2
-uset_removeAllCodePoints(USet *set, const UChar *str, int32_t length) {
+uset_removeAllCodePoints(USet *set, const char16_t *str, int32_t length) {
     UnicodeString s(length==-1, str, length);
     ((UnicodeSet*) set)->UnicodeSet::removeAll(s);
 }
@@ -133,13 +134,13 @@ uset_retain(USet* set, UChar32 start, UChar32 end) {
 }
 
 U_CAPI void U_EXPORT2
-uset_retainString(USet *set, const UChar *str, int32_t length) {
+uset_retainString(USet *set, const char16_t *str, int32_t length) {
     UnicodeString s(length==-1, str, length);
     ((UnicodeSet*) set)->UnicodeSet::retain(s);
 }
 
 U_CAPI void U_EXPORT2
-uset_retainAllCodePoints(USet *set, const UChar *str, int32_t length) {
+uset_retainAllCodePoints(USet *set, const char16_t *str, int32_t length) {
     UnicodeString s(length==-1, str, length);
     ((UnicodeSet*) set)->UnicodeSet::retainAll(s);
 }
@@ -165,13 +166,13 @@ uset_complementRange(USet *set, UChar32 start, UChar32 end) {
 }
 
 U_CAPI void U_EXPORT2
-uset_complementString(USet *set, const UChar *str, int32_t length) {
+uset_complementString(USet *set, const char16_t *str, int32_t length) {
     UnicodeString s(length==-1, str, length);
     ((UnicodeSet*) set)->UnicodeSet::complement(s);
 }
 
 U_CAPI void U_EXPORT2
-uset_complementAllCodePoints(USet *set, const UChar *str, int32_t length) {
+uset_complementAllCodePoints(USet *set, const char16_t *str, int32_t length) {
     UnicodeString s(length==-1, str, length);
     ((UnicodeSet*) set)->UnicodeSet::complementAll(s);
 }
@@ -212,7 +213,7 @@ uset_containsRange(const USet* set, UChar32 start, UChar32 end) {
 }
 
 U_CAPI UBool U_EXPORT2
-uset_containsString(const USet* set, const UChar* str, int32_t strLen) {
+uset_containsString(const USet* set, const char16_t* str, int32_t strLen) {
     UnicodeString s(strLen==-1, str, strLen);
     return ((const UnicodeSet*) set)->UnicodeSet::contains(s);
 }
@@ -223,7 +224,7 @@ uset_containsAll(const USet* set1, const USet* set2) {
 }
 
 U_CAPI UBool U_EXPORT2
-uset_containsAllCodePoints(const USet* set, const UChar *str, int32_t strLen) {
+uset_containsAllCodePoints(const USet* set, const char16_t *str, int32_t strLen) {
     // Create a string alias, since nothing is being added to the set.
     UnicodeString s(strLen==-1, str, strLen);
     return ((const UnicodeSet*) set)->UnicodeSet::containsAll(s);
@@ -240,12 +241,12 @@ uset_containsSome(const USet* set1, const USet* set2) {
 }
 
 U_CAPI int32_t U_EXPORT2
-uset_span(const USet *set, const UChar *s, int32_t length, USetSpanCondition spanCondition) {
+uset_span(const USet *set, const char16_t *s, int32_t length, USetSpanCondition spanCondition) {
     return ((UnicodeSet*) set)->UnicodeSet::span(s, length, spanCondition);
 }
 
 U_CAPI int32_t U_EXPORT2
-uset_spanBack(const USet *set, const UChar *s, int32_t length, USetSpanCondition spanCondition) {
+uset_spanBack(const USet *set, const char16_t *s, int32_t length, USetSpanCondition spanCondition) {
     return ((UnicodeSet*) set)->UnicodeSet::spanBack(s, length, spanCondition);
 }
 
@@ -306,16 +307,36 @@ uset_getRangeCount(const USet *set) {
     return ((const UnicodeSet *)set)->UnicodeSet::getRangeCount();
 }
 
+U_CAPI int32_t U_EXPORT2
+uset_getStringCount(const USet *uset) {
+    const UnicodeSet &set = *(const UnicodeSet *)uset;
+    return USetAccess::getStringCount(set);
+}
+
 U_CAPI int32_t U_EXPORT2
 uset_getItemCount(const USet* uset) {
     const UnicodeSet& set = *(const UnicodeSet*)uset;
     return set.getRangeCount() + USetAccess::getStringCount(set);
 }
 
+U_CAPI const UChar* U_EXPORT2
+uset_getString(const USet *uset, int32_t index, int32_t *pLength) {
+    if (pLength == nullptr) { return nullptr; }
+    const UnicodeSet &set = *(const UnicodeSet *)uset;
+    int32_t count = USetAccess::getStringCount(set);
+    if (index < 0 || count <= index) {
+        *pLength = 0;
+        return nullptr;
+    }
+    const UnicodeString *s = USetAccess::getString(set, index);
+    *pLength = s->length();
+    return toUCharPtr(s->getBuffer());
+}
+
 U_CAPI int32_t U_EXPORT2
 uset_getItem(const USet* uset, int32_t itemIndex,
              UChar32* start, UChar32* end,
-             UChar* str, int32_t strCapacity,
+             char16_t* str, int32_t strCapacity,
              UErrorCode* ec) {
     if (U_FAILURE(*ec)) return 0;
     const UnicodeSet& set = *(const UnicodeSet*)uset;
@@ -372,7 +393,7 @@ uset_getItem(const USet* uset, int32_t itemIndex,
  */
 U_CAPI int32_t U_EXPORT2
 uset_serialize(const USet* set, uint16_t* dest, int32_t destCapacity, UErrorCode* ec) {
-    if (ec==NULL || U_FAILURE(*ec)) {
+    if (ec==nullptr || U_FAILURE(*ec)) {
         return 0;
     }
 
@@ -383,10 +404,10 @@ U_CAPI UBool U_EXPORT2
 uset_getSerializedSet(USerializedSet* fillSet, const uint16_t* src, int32_t srcLength) {
     int32_t length;
 
-    if(fillSet==NULL) {
+    if(fillSet==nullptr) {
         return false;
     }
-    if(src==NULL || srcLength<=0) {
+    if(src==nullptr || srcLength<=0) {
         fillSet->length=fillSet->bmpLength=0;
         return false;
     }
@@ -415,7 +436,7 @@ uset_getSerializedSet(USerializedSet* fillSet, const uint16_t* src, int32_t srcL
 
 U_CAPI void U_EXPORT2
 uset_setSerializedToOne(USerializedSet* fillSet, UChar32 c) {
-    if(fillSet==NULL || (uint32_t)c>0x10ffff) {
+    if(fillSet==nullptr || (uint32_t)c>0x10ffff) {
         return;
     }
 
@@ -450,7 +471,7 @@ U_CAPI UBool U_EXPORT2
 uset_serializedContains(const USerializedSet* set, UChar32 c) {
     const uint16_t* array;
 
-    if(set==NULL || (uint32_t)c>0x10ffff) {
+    if(set==nullptr || (uint32_t)c>0x10ffff) {
         return false;
     }
 
@@ -475,7 +496,7 @@ uset_serializedContains(const USerializedSet* set, UChar32 c) {
         } else {
             hi += 1;
         }
-        return (UBool)(hi&1);
+        return hi&1;
     } else {
         /* find c in the supplementary part */
         uint16_t high=(uint16_t)(c>>16), low=(uint16_t)c;
@@ -500,13 +521,13 @@ uset_serializedContains(const USerializedSet* set, UChar32 c) {
             hi += 2;
         }
         /* count pairs of 16-bit units even per BMP and check if the number of pairs is odd */
-        return (UBool)(((hi+(base<<1))&2)!=0);
+        return ((hi+(base<<1))&2)!=0;
     }
 }
 
 U_CAPI int32_t U_EXPORT2
 uset_getSerializedRangeCount(const USerializedSet* set) {
-    if(set==NULL) {
+    if(set==nullptr) {
         return 0;
     }
 
@@ -519,7 +540,7 @@ uset_getSerializedRange(const USerializedSet* set, int32_t rangeIndex,
     const uint16_t* array;
     int32_t bmpLength, length;
 
-    if(set==NULL || rangeIndex<0 || pStart==NULL || pEnd==NULL) {
+    if(set==nullptr || rangeIndex<0 || pStart==nullptr || pEnd==nullptr) {
         return false;
     }
 
@@ -590,7 +611,7 @@ uset_getSerializedRange(const USerializedSet* set, int32_t rangeIndex,
 // addRemove(USet* set, UChar32 c, int32_t doRemove) {
 //     int32_t i, length, more;
 // 
-//     if(set==NULL || (uint32_t)c>0x10ffff) {
+//     if(set==nullptr || (uint32_t)c>0x10ffff) {
 //         return false;
 //     }
 // 
@@ -646,7 +667,7 @@ uset_getSerializedRange(const USerializedSet* set, int32_t rangeIndex,
 //         /* reallocate */
 //         int32_t newCapacity=set->capacity+set->capacity/2+USET_GROW_DELTA;
 //         UChar32* newArray=(UChar32* )uprv_malloc(newCapacity*4);
-//         if(newArray==NULL) {
+//         if(newArray==nullptr) {
 //             return false;
 //         }
 //         set->capacity=newCapacity;
