@@ -51,6 +51,7 @@ static void TestNarrowQuarters(void);
 static void TestExtraneousCharacters(void);
 static void TestParseTooStrict(void);
 static void TestHourCycle(void);
+static void TestLocaleNameCrash(void);
 
 void addDateForTest(TestNode** root);
 
@@ -76,9 +77,10 @@ void addDateForTest(TestNode** root)
     TESTCASE(TestExtraneousCharacters);
     TESTCASE(TestParseTooStrict);
     TESTCASE(TestHourCycle);
+    TESTCASE(TestLocaleNameCrash);
 }
 /* Testing the DateFormat API */
-static void TestDateFormat()
+static void TestDateFormat(void)
 {
     UDateFormat *def, *fr, *it, *de, *def1, *fr_pat;
     UDateFormat *any;
@@ -438,7 +440,7 @@ enum { kDateOrTimeOutMax = 96, kDateAndTimeOutMax = 192 };
 static const UDate minutesTolerance = 2 * 60.0 * 1000.0;
 static const UDate daysTolerance = 2 * 24.0 * 60.0 * 60.0 * 1000.0;
 
-static void TestRelativeDateFormat()
+static void TestRelativeDateFormat(void)
 {
     UDate today = 0.0;
     const UDateFormatStyle * stylePtr;
@@ -588,9 +590,9 @@ static void TestRelativeDateFormat()
 }
 
 /*Testing udat_getSymbols() and udat_setSymbols() and udat_countSymbols()*/
-static void TestSymbols()
+static void TestSymbols(void)
 {
-    UDateFormat *def, *fr, *zhChiCal, *esMX;
+    UDateFormat *def, *fr, *zhChiCal, *esMX, *th;
     UErrorCode status = U_ZERO_ERROR;
     UChar *value=NULL;
     UChar *result = NULL;
@@ -638,6 +640,15 @@ static void TestSymbols()
             myErrorName(status) );
         return;
     }
+    /*creating a dateformat with th locale */
+    log_verbose("\ncreating a date format with th locale\n");
+    th = udat_open(UDAT_SHORT, UDAT_NONE, "th", NULL, 0, NULL, 0, &status);
+    if(U_FAILURE(status))
+    {
+        log_data_err("error in creating the dateformat using no date, short time, locale th -> %s (Are you missing data?)\n",
+            myErrorName(status) );
+        return;
+    }
 
     /*Testing countSymbols, getSymbols and setSymbols*/
     log_verbose("\nTesting countSymbols\n");
@@ -708,6 +719,10 @@ static void TestSymbols()
     VerifygetSymbols(zhChiCal, UDAT_CYCLIC_YEARS_NARROW, 59, "\\u7678\\u4EA5");
     VerifygetSymbols(zhChiCal, UDAT_ZODIAC_NAMES_ABBREVIATED, 0, "\\u9F20");
     VerifygetSymbols(zhChiCal, UDAT_ZODIAC_NAMES_WIDE, 11, "\\u732A");
+    VerifygetSymbols(def, UDAT_AM_PMS_NARROW, 0, "a");
+    VerifygetSymbols(def, UDAT_AM_PMS_NARROW, 1, "p");
+    VerifygetSymbols(th, UDAT_AM_PMS_WIDE, 0, "\\u0E01\\u0E48\\u0E2D\\u0E19\\u0E40\\u0E17\\u0E35\\u0E48\\u0E22\\u0E07");
+    VerifygetSymbols(th, UDAT_AM_PMS_WIDE, 1, "\\u0E2B\\u0E25\\u0E31\\u0E07\\u0E40\\u0E17\\u0E35\\u0E48\\u0E22\\u0E07");
 #if UDAT_HAS_PATTERN_CHAR_FOR_TIME_SEPARATOR
     VerifygetSymbols(def,UDAT_LOCALIZED_CHARS, 0, "GyMdkHmsSEDFwWahKzYeugAZvcLQqVUOXxrbB:");
 #else
@@ -851,6 +866,7 @@ free(pattern);
     udat_close(def);
     udat_close(zhChiCal);
     udat_close(esMX);
+    udat_close(th);
     if(result != NULL) {
         free(result);
         result = NULL;
@@ -862,7 +878,7 @@ free(pattern);
 /**
  * Test DateFormat(Calendar) API
  */
-static void TestDateFormatCalendar() {
+static void TestDateFormatCalendar(void) {
     UDateFormat *date=0, *time=0, *full=0;
     UCalendar *cal=0;
     UChar buf[256];
@@ -989,7 +1005,7 @@ static void TestDateFormatCalendar() {
 /**
  * Test parsing two digit year against "YY" vs. "YYYY" patterns
  */
-static void TestCalendarDateParse() {
+static void TestCalendarDateParse(void) {
 
     int32_t result;
     UErrorCode ec = U_ZERO_ERROR;
@@ -1069,7 +1085,7 @@ static void TestCalendarDateParse() {
 
 
 /*INTERNAL FUNCTIONS USED*/
-static int getCurrentYear() {
+static int getCurrentYear(void) {
     static int currentYear = 0;
     if (currentYear == 0) {
         UErrorCode status = U_ZERO_ERROR;
@@ -1302,7 +1318,7 @@ static UBool _aux2ExtremeDates(UDateFormat* fmt, UDate small, UDate large,
  *  0.75*10^30, etc.  A logarithmic search will find 10^15, then 10^7.5
  *  and 10^22.5, etc.
  */
-static void TestExtremeDates() {
+static void TestExtremeDates(void) {
     UDateFormat *fmt;
     UErrorCode ec;
     UChar buf[256];
@@ -2087,8 +2103,8 @@ static void TestHourCycle(void) {
         // test some locales for which we have data
         u"en_US", u"Tuesday, March 16, 1943 at 3:45:32 PM",
         u"en_CA", u"Tuesday, March 16, 1943 at 3:45:32 p.m.",
-        u"en_GB", u"Tuesday 16 March 1943 at 15:45:32",
-        u"en_AU", u"Tuesday 16 March 1943 at 3:45:32 pm",
+        u"en_GB", u"Tuesday, 16 March 1943 at 15:45:32",
+        u"en_AU", u"Tuesday, 16 March 1943 at 3:45:32 pm",
         // test a couple locales for which we don't have specific locale files (we should still get the correct hour cycle)
         u"en_CO", u"Tuesday, March 16, 1943 at 3:45:32 PM",
         u"en_MX", u"Tuesday, March 16, 1943 at 3:45:32 PM",
@@ -2096,15 +2112,15 @@ static void TestHourCycle(void) {
         u"en_US@rg=GBzzzz", u"Tuesday, March 16, 1943 at 15:45:32",
         u"en_US@rg=CAzzzz", u"Tuesday, March 16, 1943 at 3:45:32 PM",
         u"en_CA@rg=USzzzz", u"Tuesday, March 16, 1943 at 3:45:32 p.m.",
-        u"en_GB@rg=USzzzz", u"Tuesday 16 March 1943 at 3:45:32 pm",
-        u"en_GB@rg=CAzzzz", u"Tuesday 16 March 1943 at 3:45:32 pm",
-        u"en_GB@rg=AUzzzz", u"Tuesday 16 March 1943 at 3:45:32 pm",
+        u"en_GB@rg=USzzzz", u"Tuesday, 16 March 1943 at 3:45:32 pm",
+        u"en_GB@rg=CAzzzz", u"Tuesday, 16 March 1943 at 3:45:32 pm",
+        u"en_GB@rg=AUzzzz", u"Tuesday, 16 March 1943 at 3:45:32 pm",
         // test that the hc ("hours") subtag does the right thing
         u"en_US@hours=h23", u"Tuesday, March 16, 1943 at 15:45:32",
-        u"en_GB@hours=h12", u"Tuesday 16 March 1943 at 3:45:32 pm",
+        u"en_GB@hours=h12", u"Tuesday, 16 March 1943 at 3:45:32 pm",
         // test that the rg and hc subtags do the right thing when used together
         u"en_US@rg=GBzzzz;hours=h12", u"Tuesday, March 16, 1943 at 3:45:32 PM",
-        u"en_GB@rg=USzzzz;hours=h23", u"Tuesday 16 March 1943 at 15:45:32",
+        u"en_GB@rg=USzzzz;hours=h23", u"Tuesday, 16 March 1943 at 15:45:32",
     };
     
     for (int32_t i = 0; i < UPRV_LENGTHOF(testCases); i += 2) {
@@ -2129,4 +2145,17 @@ static void TestHourCycle(void) {
     }
 }
 
+static void TestLocaleNameCrash(void) {
+    UErrorCode status = U_ZERO_ERROR;
+    UDateFormat icudf;
+
+    icudf = udat_open(UDAT_MEDIUM, UDAT_NONE, "notalanguage", NULL, 0, NULL, 0, &status);
+    if ( U_SUCCESS(status) ) {
+        log_verbose("Success: did not crash on udat_open(locale=\"notalanguage\")\n");
+    } else {
+        log_err("FAIL: didn't crash on udat_open(locale=\"notalanguage\"), but got %s.\n", u_errorName(status));
+    }
+    udat_close(icudf);
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
