@@ -15,6 +15,7 @@
  *   created on: 2003jul11
  *   created by: Ram Viswanadha
  */
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include "unicode/utypes.h"
@@ -79,7 +80,7 @@ testAPI(const UChar* src, const UChar* expected, const char* testName,
     int32_t destLen = 0;
     UChar* dest = NULL;
     int32_t expectedLen = (expected != NULL) ? u_strlen(expected) : 0;
-    int32_t options = (useSTD3ASCIIRules == TRUE) ? UIDNA_USE_STD3_RULES : UIDNA_DEFAULT;
+    int32_t options = (useSTD3ASCIIRules == true) ? UIDNA_USE_STD3_RULES : UIDNA_DEFAULT;
     UParseError parseError;
     int32_t tSrcLen = 0;
     UChar* tSrc = NULL;
@@ -99,7 +100,7 @@ testAPI(const UChar* src, const UChar* expected, const char* testName,
             dest = destStack;
             destLen = func(src,-1,dest,destLen+1,options, &parseError, &status);
             /* TODO : compare output with expected */
-            if(U_SUCCESS(status) && expectedStatus != U_IDNA_STD3_ASCII_RULES_ERROR&& (doCompare==TRUE) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
+            if(U_SUCCESS(status) && expectedStatus != U_IDNA_STD3_ASCII_RULES_ERROR&& (doCompare==true) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
                 log_err("Did not get the expected result for  null terminated source.\n" );
             }
         }else{
@@ -121,7 +122,7 @@ testAPI(const UChar* src, const UChar* expected, const char* testName,
                 dest = destStack;
                 destLen = func(src,-1,dest,destLen+1,options | UIDNA_ALLOW_UNASSIGNED, &parseError, &status);
                 /* TODO : compare output with expected */
-                if(U_SUCCESS(status) && (doCompare==TRUE) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
+                if(U_SUCCESS(status) && (doCompare==true) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
                     log_err("Did not get the expected result for %s null terminated source with both options set.\n",testName);
 
                 }
@@ -145,7 +146,7 @@ testAPI(const UChar* src, const UChar* expected, const char* testName,
             dest = destStack;
             destLen = func(src,u_strlen(src),dest,destLen+1,options, &parseError, &status);
             /* TODO : compare output with expected */
-            if(U_SUCCESS(status) && (doCompare==TRUE) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
+            if(U_SUCCESS(status) && (doCompare==true) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
                 log_err("Did not get the expected result for %s with source length.\n",testName);
             }
         }else{
@@ -167,7 +168,7 @@ testAPI(const UChar* src, const UChar* expected, const char* testName,
                 dest = destStack;
                 destLen = func(src,u_strlen(src),dest,destLen+1,options | UIDNA_ALLOW_UNASSIGNED, &parseError, &status);
                 /* TODO : compare output with expected */
-                if(U_SUCCESS(status) && (doCompare==TRUE) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
+                if(U_SUCCESS(status) && (doCompare==true) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
                     log_err("Did not get the expected result for %s with source length and both options set.\n",testName);
                 }
             }else{
@@ -188,7 +189,7 @@ testAPI(const UChar* src, const UChar* expected, const char* testName,
             dest = destStack;
             destLen = func(src,-1,dest,destLen+1,options | UIDNA_USE_STD3_RULES, &parseError, &status);
             /* TODO : compare output with expected*/
-            if(U_SUCCESS(status) && (doCompare==TRUE) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
+            if(U_SUCCESS(status) && (doCompare==true) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
                 log_err("Did not get the expected result for %s null terminated source with both options set.\n",testName);
 
             }
@@ -211,7 +212,7 @@ testAPI(const UChar* src, const UChar* expected, const char* testName,
             dest = destStack;
             destLen = func(src,u_strlen(src),dest,destLen+1,options | UIDNA_USE_STD3_RULES, &parseError, &status);
             /* TODO : compare output with expected*/
-            if(U_SUCCESS(status) && (doCompare==TRUE) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
+            if(U_SUCCESS(status) && (doCompare==true) && u_strCaseCompare(dest,destLen, expected,expectedLen,0,&status)!=0){
                 log_err("Did not get the expected result for %s with source length and both options set.\n",testName);
             }
         }else{
@@ -417,7 +418,7 @@ static const char * const domainNames[] = {
     /*"www.\\u0021.com",*/
     /*"www.\\u0024.com",*/
     /*"\\u003f",*/
-    /* These yeild U_IDNA_PROHIBITED_ERROR*/
+    /* These yield U_IDNA_PROHIBITED_ERROR*/
     /*"\\u00CF\\u0082.com",*/
     /*"\\u00CE\\u00B2\\u00C3\\u009Fss.com",*/
     /*"\\u00E2\\u0098\\u00BA.com",*/
@@ -426,7 +427,7 @@ static const char * const domainNames[] = {
 };
 
 static void
-TestToASCII(){
+TestToASCII(void){
 
     int32_t i;
     UChar buf[MAX_DEST_SIZE];
@@ -434,13 +435,13 @@ TestToASCII(){
     TestFunc func = uidna_toASCII;
     for(i=0;i< UPRV_LENGTHOF(unicodeIn); i++){
         u_charsToUChars(asciiIn[i],buf, (int32_t)strlen(asciiIn[i])+1);
-        testAPI(unicodeIn[i], buf,testName, FALSE,U_ZERO_ERROR, TRUE, TRUE, func);
+        testAPI(unicodeIn[i], buf,testName, false,U_ZERO_ERROR, true, true, func);
 
     }
 }
 
 static void
-TestToUnicode(){
+TestToUnicode(void){
 
     int32_t i;
     UChar buf[MAX_DEST_SIZE];
@@ -448,13 +449,13 @@ TestToUnicode(){
     TestFunc func = uidna_toUnicode;
     for(i=0;i< UPRV_LENGTHOF(asciiIn); i++){
         u_charsToUChars(asciiIn[i],buf, (int32_t)strlen(asciiIn[i])+1);
-        testAPI(buf,unicodeIn[i],testName,FALSE,U_ZERO_ERROR, TRUE, TRUE, func);
+        testAPI(buf,unicodeIn[i],testName,false,U_ZERO_ERROR, true, true, func);
     }
 }
 
 
 static void
-TestIDNToUnicode(){
+TestIDNToUnicode(void){
     int32_t i;
     UChar buf[MAX_DEST_SIZE];
     UChar expected[MAX_DEST_SIZE];
@@ -471,9 +472,9 @@ TestIDNToUnicode(){
             log_err_status(status,  "%s failed to convert domainNames[%i].Error: %s \n",testName, i, u_errorName(status));
             break;
         }
-        testAPI(buf,expected,testName,FALSE,U_ZERO_ERROR, TRUE, TRUE, func);
+        testAPI(buf,expected,testName,false,U_ZERO_ERROR, true, true, func);
          /*test toUnicode with all labels in the string*/
-        testAPI(buf,expected,testName, FALSE,U_ZERO_ERROR, TRUE, TRUE, func);
+        testAPI(buf,expected,testName, false,U_ZERO_ERROR, true, true, func);
         if(U_FAILURE(status)){
             log_err( "%s failed to convert domainNames[%i].Error: %s \n",testName,i, u_errorName(status));
             break;
@@ -483,7 +484,7 @@ TestIDNToUnicode(){
 }
 
 static void
-TestIDNToASCII(){
+TestIDNToASCII(void){
     int32_t i;
     UChar buf[MAX_DEST_SIZE];
     UChar expected[MAX_DEST_SIZE];
@@ -501,9 +502,9 @@ TestIDNToASCII(){
             log_err_status(status,  "%s failed to convert domainNames[%i].Error: %s \n",testName,i, u_errorName(status));
             break;
         }
-        testAPI(buf,expected,testName, FALSE,U_ZERO_ERROR, TRUE, TRUE, func);
+        testAPI(buf,expected,testName, false,U_ZERO_ERROR, true, true, func);
         /*test toASCII with all labels in the string*/
-        testAPI(buf,expected,testName, FALSE,U_ZERO_ERROR, FALSE, TRUE, func);
+        testAPI(buf,expected,testName, false,U_ZERO_ERROR, false, true, func);
         if(U_FAILURE(status)){
             log_err( "%s failed to convert domainNames[%i].Error: %s \n",testName,i, u_errorName(status));
             break;
@@ -523,7 +524,7 @@ testCompareWithSrc(const UChar* s1, int32_t s1Len,
     UErrorCode status = U_ZERO_ERROR;
     int32_t retVal = func(s1,-1,s2,-1,UIDNA_DEFAULT,&status);
 
-    if(isEqual==TRUE &&  retVal !=0){
+    if(isEqual==true &&  retVal !=0){
         log_err("Did not get the expected result for %s with null termniated strings.\n",testName);
     }
     if(U_FAILURE(status)){
@@ -533,7 +534,7 @@ testCompareWithSrc(const UChar* s1, int32_t s1Len,
     status = U_ZERO_ERROR;
     retVal = func(s1,-1,s2,-1,UIDNA_ALLOW_UNASSIGNED,&status);
 
-    if(isEqual==TRUE &&  retVal !=0){
+    if(isEqual==true &&  retVal !=0){
         log_err("Did not get the expected result for %s with null termniated strings with options set.\n", testName);
     }
     if(U_FAILURE(status)){
@@ -543,7 +544,7 @@ testCompareWithSrc(const UChar* s1, int32_t s1Len,
     status = U_ZERO_ERROR;
     retVal = func(s1,s1Len,s2,s2Len,UIDNA_DEFAULT,&status);
 
-    if(isEqual==TRUE &&  retVal !=0){
+    if(isEqual==true &&  retVal !=0){
         log_err("Did not get the expected result for %s with string length.\n",testName);
     }
     if(U_FAILURE(status)){
@@ -553,7 +554,7 @@ testCompareWithSrc(const UChar* s1, int32_t s1Len,
     status = U_ZERO_ERROR;
     retVal = func(s1,s1Len,s2,s2Len,UIDNA_ALLOW_UNASSIGNED,&status);
 
-    if(isEqual==TRUE &&  retVal !=0){
+    if(isEqual==true &&  retVal !=0){
         log_err("Did not get the expected result for %s with string length and options set.\n",testName);
     }
     if(U_FAILURE(status)){
@@ -563,7 +564,7 @@ testCompareWithSrc(const UChar* s1, int32_t s1Len,
 
 
 static void
-TestCompare(){
+TestCompare(void){
     int32_t i;
 
     const char* testName ="uidna_compare";
@@ -619,28 +620,28 @@ TestCompare(){
         src = source;
         srcLen = u_strlen(src);
 
-        testCompareWithSrc(src,srcLen,src,srcLen,testName, func, TRUE);
+        testCompareWithSrc(src,srcLen,src,srcLen,testName, func, true);
 
         /* b) compare it with asciiIn equivalent */
-        testCompareWithSrc(src,srcLen,buf,u_strlen(buf),testName, func,TRUE);
+        testCompareWithSrc(src,srcLen,buf,u_strlen(buf),testName, func,true);
 
         /* c) compare it with unicodeIn not equivalent*/
         if(i==0){
-            testCompareWithSrc(src,srcLen,uni1,u_strlen(uni1),testName, func,FALSE);
+            testCompareWithSrc(src,srcLen,uni1,u_strlen(uni1),testName, func,false);
         }else{
-            testCompareWithSrc(src,srcLen,uni0,u_strlen(uni0),testName, func,FALSE);
+            testCompareWithSrc(src,srcLen,uni0,u_strlen(uni0),testName, func,false);
         }
         /* d) compare it with asciiIn not equivalent */
         if(i==0){
-            testCompareWithSrc(src,srcLen,ascii1,u_strlen(ascii1),testName, func,FALSE);
+            testCompareWithSrc(src,srcLen,ascii1,u_strlen(ascii1),testName, func,false);
         }else{
-            testCompareWithSrc(src,srcLen,ascii0,u_strlen(ascii0),testName, func,FALSE);
+            testCompareWithSrc(src,srcLen,ascii0,u_strlen(ascii0),testName, func,false);
         }
 
     }
 }
 
-static void TestJB4490(){
+static void TestJB4490(void){
     static const UChar data[][50]= {
         {0x00F5,0x00dE,0x00dF,0x00dD, 0x0000},
         {0xFB00,0xFB01}
@@ -672,7 +673,7 @@ static void TestJB4490(){
     }
 }
 
-static void TestJB4475(){
+static void TestJB4475(void){
     
     static const UChar input[][10] = {
         {0x0054,0x0045,0x0053,0x0054,0x0000},/* TEST */
@@ -699,7 +700,7 @@ static void TestJB4475(){
     }
 }
 
-static void TestLength(){
+static void TestLength(void){
     {
         static const char* cl = "my_very_very_very_very_very_very_very_very_very_very_very_very_very_long_and_incredibly_uncreative_domain_label";
         UChar ul[128] = {'\0'};
@@ -822,7 +823,7 @@ static void TestLength(){
         }
     }    
 }
-static void TestJB5273(){
+static void TestJB5273(void){
     static const char INVALID_DOMAIN_NAME[] = "xn--m\\u00FCller.de";
     UChar invalid_idn[25] = {'\0'};
     int32_t len = u_unescape(INVALID_DOMAIN_NAME, invalid_idn, (int32_t)strlen(INVALID_DOMAIN_NAME));
@@ -856,7 +857,7 @@ static void TestJB5273(){
  * Test the new (ICU 4.6/2010) C API that was added for UTS #46.
  * Just an API test: Functionality is tested via C++ intltest.
  */
-static void TestUTS46() {
+static void TestUTS46(void) {
     static const UChar fA_sharps16[] = { 0x66, 0x41, 0xdf, 0 };
     static const char fA_sharps8[] = { 0x66, 0x41, (char)0xc3, (char)0x9f, 0 };
     static const UChar fa_sharps16[] = { 0x66, 0x61, 0xdf, 0 };
