@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
- *   Copyright (C) 1997-2009,2014 International Business Machines
+ *   Copyright (C) 1997-2016 International Business Machines
  *   Corporation and others.  All Rights Reserved.
  *******************************************************************************
  *   Date        Name        Description
@@ -12,12 +14,14 @@
 
 #if !UCONFIG_NO_TRANSLITERATION
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include "unicode/utrans.h"
 #include "unicode/ustring.h"
 #include "unicode/uset.h"
 #include "cintltst.h"
+#include "cmemory.h"
 
 #define TEST(x) addTest(root, &x, "utrans/" # x)
 
@@ -30,6 +34,7 @@ static void TestRegisterUnregister(void);
 static void TestExtractBetween(void);
 static void TestUnicodeIDs(void);
 static void TestGetRulesAndSourceSet(void);
+static void TestDataVariantsCompounds(void);
 
 static void _expectRules(const char*, const char*, const char*);
 static void _expect(const UTransliterator* trans, const char* cfrom, const char* cto);
@@ -48,6 +53,7 @@ addUTransTest(TestNode** root) {
     TEST(TestExtractBetween);
     TEST(TestUnicodeIDs);
     TEST(TestGetRulesAndSourceSet);
+    TEST(TestDataVariantsCompounds);
 }
 
 /*------------------------------------------------------------------
@@ -136,7 +142,7 @@ static void InitXReplaceableCallbacks(UReplaceableCallbacks* callbacks) {
  * Tests
  *------------------------------------------------------------------*/
 
-static void TestAPI() {
+static void TestAPI(void) {
     enum { BUF_CAP = 128 };
     char buf[BUF_CAP], buf2[BUF_CAP];
     UErrorCode status = U_ZERO_ERROR;
@@ -178,7 +184,7 @@ static void TestAPI() {
     }
 }
 
-static void TestUnicodeIDs() {
+static void TestUnicodeIDs(void) {
     UEnumeration *uenum;
     UTransliterator *utrans;
     const UChar *id, *id2;
@@ -243,7 +249,7 @@ static void TestUnicodeIDs() {
     uenum_close(uenum);
 }
 
-static void TestOpenInverse(){
+static void TestOpenInverse(void){
     UErrorCode status=U_ZERO_ERROR;
     UTransliterator* t1=NULL;
     UTransliterator* inverse1=NULL;
@@ -270,7 +276,7 @@ static void TestOpenInverse(){
            "Hex-Any"
          };
      
-    for(i=0; i<sizeof(TransID)/sizeof(TransID[0]); i=i+2){
+    for(i=0; i<UPRV_LENGTHOF(TransID); i=i+2){
         status = U_ZERO_ERROR;
         t1=utrans_open(TransID[i], UTRANS_FORWARD,NULL,0,NULL, &status);
         if(t1 == NULL || U_FAILURE(status)){
@@ -291,7 +297,7 @@ static void TestOpenInverse(){
    }
 }
 
-static void TestClone(){
+static void TestClone(void){
     UErrorCode status=U_ZERO_ERROR;
     UTransliterator* t1=NULL;
     UTransliterator* t2=NULL;
@@ -338,7 +344,7 @@ static void TestClone(){
 
 }
 
-static void TestRegisterUnregister(){
+static void TestRegisterUnregister(void){
     UErrorCode status=U_ZERO_ERROR;
     UTransliterator* t1=NULL;
     UTransliterator* rules=NULL, *rules2;
@@ -433,12 +439,12 @@ static void TestRegisterUnregister(){
     utrans_close(inverse1);
 }
 
-static void TestSimpleRules() {
+static void TestSimpleRules(void) {
     /* Test rules */
     /* Example: rules 1. ab>x|y
      *                2. yc>z
      *
-     * []|eabcd  start - no match, copy e to tranlated buffer
+     * []|eabcd  start - no match, copy e to translated buffer
      * [e]|abcd  match rule 1 - copy output & adjust cursor
      * [ex|y]cd  match rule 2 - copy output & adjust cursor
      * [exz]|d   no match, copy d to transliterated buffer
@@ -486,7 +492,7 @@ static void TestSimpleRules() {
                  "abc ababc aba", "xy abxy z"); 
 }
 
-static void TestFilter() {
+static void TestFilter(void) {
     UErrorCode status = U_ZERO_ERROR;
     UChar filt[128];
     UChar buf[128];
@@ -502,7 +508,7 @@ static void TestFilter() {
         "abcde",
         "\\u0061\\u0062\\u0063\\u0064\\u0065"
     };
-    int32_t DATA_length = sizeof(DATA) / sizeof(DATA[0]);
+    int32_t DATA_length = UPRV_LENGTHOF(DATA);
     int32_t i;
 
     UTransliterator* hex = utrans_open("Any-Hex", UTRANS_FORWARD, NULL,0,NULL,&status);
@@ -552,7 +558,7 @@ static void TestFilter() {
  * Test the UReplaceableCallback extractBetween support.  We use a
  * transliterator known to rely on this call.
  */
-static void TestExtractBetween() {
+static void TestExtractBetween(void) {
 
     UTransliterator *trans;
     UErrorCode status = U_ZERO_ERROR;
@@ -580,8 +586,8 @@ static void TestExtractBetween() {
 static const UChar transSimpleID[] = { 0x79,0x6F,0x2D,0x79,0x6F,0x5F,0x42,0x4A,0 }; /* "yo-yo_BJ" */
 static const char* transSimpleCName = "yo-yo_BJ";
 
-enum { kUBufMax = 256 };
-static void TestGetRulesAndSourceSet() {
+enum { kUBufMax = 512 };
+static void TestGetRulesAndSourceSet(void) {
     UErrorCode status = U_ZERO_ERROR;
     UTransliterator *utrans = utrans_openU(transSimpleID, -1, UTRANS_FORWARD, NULL, 0, NULL, &status);
     if ( U_SUCCESS(status) ) {
@@ -590,29 +596,29 @@ static void TestGetRulesAndSourceSet() {
         int32_t ulen;
 
         status = U_ZERO_ERROR;
-        ulen = utrans_toRules(utrans, FALSE, ubuf, kUBufMax, &status);
+        ulen = utrans_toRules(utrans, false, ubuf, kUBufMax, &status);
         if ( U_FAILURE(status) || ulen <= 50 || ulen >= 100) {
             log_err("FAIL: utrans_toRules unescaped, expected noErr and len 50-100, got error=%s and len=%d\n",
                     u_errorName(status), ulen);
         }
 
         status = U_ZERO_ERROR;
-        ulen = utrans_toRules(utrans, FALSE, NULL, 0, &status);
+        ulen = utrans_toRules(utrans, false, NULL, 0, &status);
         if ( status != U_BUFFER_OVERFLOW_ERROR || ulen <= 50 || ulen >= 100) {
             log_err("FAIL: utrans_toRules unescaped, expected U_BUFFER_OVERFLOW_ERROR and len 50-100, got error=%s and len=%d\n",
                     u_errorName(status), ulen);
         }
 
         status = U_ZERO_ERROR;
-        ulen = utrans_toRules(utrans, TRUE, ubuf, kUBufMax, &status);
+        ulen = utrans_toRules(utrans, true, ubuf, kUBufMax, &status);
         if ( U_FAILURE(status) || ulen <= 100 || ulen >= 200) {
             log_err("FAIL: utrans_toRules escaped, expected noErr and len 100-200, got error=%s and len=%d\n",
                     u_errorName(status), ulen);
         }
 
         status = U_ZERO_ERROR;
-        uset = utrans_getSourceSet(utrans, FALSE, NULL, &status);
-        ulen = uset_toPattern(uset, ubuf, kUBufMax, FALSE, &status);
+        uset = utrans_getSourceSet(utrans, false, NULL, &status);
+        ulen = uset_toPattern(uset, ubuf, kUBufMax, false, &status);
         uset_close(uset);
         if ( U_FAILURE(status) || ulen <= 4 || ulen >= 20) {
             log_err("FAIL: utrans_getSourceSet useFilter, expected noErr and len 4-20, got error=%s and len=%d\n",
@@ -620,8 +626,8 @@ static void TestGetRulesAndSourceSet() {
         }
 
         status = U_ZERO_ERROR;
-        uset = utrans_getSourceSet(utrans, TRUE, NULL, &status);
-        ulen = uset_toPattern(uset, ubuf, kUBufMax, FALSE, &status);
+        uset = utrans_getSourceSet(utrans, true, NULL, &status);
+        ulen = uset_toPattern(uset, ubuf, kUBufMax, false, &status);
         uset_close(uset);
         if ( U_FAILURE(status) || ulen <= 4 || ulen >= 20) {
             log_err("FAIL: utrans_getSourceSet ignoreFilter, expected noErr and len 4-20, got error=%s and len=%d\n",
@@ -635,6 +641,67 @@ static void TestGetRulesAndSourceSet() {
     }
 }
 
+typedef struct {
+    const char * transID;
+    const char * sourceText;
+    const char * targetText;
+} TransIDSourceTarg;
+
+static const TransIDSourceTarg dataVarCompItems[] = {
+    { "Simplified-Traditional",
+       "\\u4E0B\\u9762\\u662F\\u4E00\\u4E9B\\u4ECE\\u7B80\\u4F53\\u8F6C\\u6362\\u4E3A\\u7E41\\u4F53\\u5B57\\u793A\\u4F8B\\u6587\\u672C\\u3002",
+       "\\u4E0B\\u9762\\u662F\\u4E00\\u4E9B\\u5F9E\\u7C21\\u9AD4\\u8F49\\u63DB\\u70BA\\u7E41\\u9AD4\\u5B57\\u793A\\u4F8B\\u6587\\u672C\\u3002" },
+    { "Halfwidth-Fullwidth",
+      "Sample text, \\uFF7B\\uFF9D\\uFF8C\\uFF9F\\uFF99\\uFF83\\uFF77\\uFF7D\\uFF84.",
+      "\\uFF33\\uFF41\\uFF4D\\uFF50\\uFF4C\\uFF45\\u3000\\uFF54\\uFF45\\uFF58\\uFF54\\uFF0C\\u3000\\u30B5\\u30F3\\u30D7\\u30EB\\u30C6\\u30AD\\u30B9\\u30C8\\uFF0E" },
+    { "Han-Latin/Names; Latin-Bopomofo",
+       "\\u4E07\\u4FDF\\u919C\\u5974\\u3001\\u533A\\u695A\\u826F\\u3001\\u4EFB\\u70E8\\u3001\\u5CB3\\u98DB",
+       "\\u3107\\u311B\\u02CB \\u3111\\u3127\\u02CA \\u3114\\u3121\\u02C7 \\u310B\\u3128\\u02CA, \\u3121 \\u3114\\u3128\\u02C7 \\u310C\\u3127\\u3124\\u02CA, \\u3116\\u3123\\u02CA \\u3127\\u311D\\u02CB, \\u3129\\u311D\\u02CB \\u3108\\u311F" },
+    { "Greek-Latin",
+      "\\u1F08 \\u1FBC \\u1F89 \\u1FEC",
+      "A \\u0100I H\\u0100I RH" },
+/* The following transform is provisional and not present in ICU 60
+    { "Greek-Latin/BGN",
+      "\\u1F08 \\u1FBC \\u1F89 \\u1FEC",
+      "A\\u0313 A\\u0345 A\\u0314\\u0345 \\u1FEC" },
+*/
+    { "Greek-Latin/UNGEGN",
+      "\\u1F08 \\u1FBC \\u1F89 \\u1FEC",
+      "A A A R" },
+    { NULL, NULL, NULL }
+};
+
+enum { kBBufMax = 1024 };
+static void TestDataVariantsCompounds(void) {
+    const TransIDSourceTarg* itemsPtr;
+    for (itemsPtr = dataVarCompItems; itemsPtr->transID != NULL; itemsPtr++) {
+        UErrorCode status = U_ZERO_ERROR;
+        UChar utrid[kUBufMax];
+        int32_t utridlen = u_unescape(itemsPtr->transID, utrid, kUBufMax);
+        UTransliterator* utrans = utrans_openU(utrid, utridlen, UTRANS_FORWARD, NULL, 0, NULL, &status);
+        if (U_FAILURE(status)) {
+            log_data_err("FAIL: utrans_openRules(%s) failed, error=%s (Are you missing data?)\n", itemsPtr->transID, u_errorName(status));
+            continue;
+        }
+        UChar text[kUBufMax];
+        int32_t textLen =  u_unescape(itemsPtr->sourceText, text, kUBufMax);
+        int32_t textLim = textLen;
+        utrans_transUChars(utrans, text, &textLen, kUBufMax, 0, &textLim, &status);
+        if (U_FAILURE(status)) {
+            log_err("FAIL: utrans_transUChars(%s) failed, error=%s\n", itemsPtr->transID, u_errorName(status));
+        } else {
+            UChar expect[kUBufMax];
+            int32_t expectLen =  u_unescape(itemsPtr->targetText, expect, kUBufMax);
+            if (textLen != expectLen || u_strncmp(text, expect, textLen) != 0) {
+                char btext[kBBufMax], bexpect[kBBufMax];
+                u_austrncpy(btext, text, kUBufMax);
+                u_austrncpy(bexpect, expect, kUBufMax);
+                log_err("FAIL: utrans_transUChars(%s),\n       expect %s\n       get    %s\n", itemsPtr->transID, bexpect, btext);
+            }
+        }
+        utrans_close(utrans);
+    }
+}
 
 static void _expectRules(const char* crules,
                   const char* cfrom,
