@@ -17,7 +17,7 @@ class DisplayOptionsTest : public IntlTest {
     void testDisplayOptionsGetIdentifier();
     void testDisplayOptionsFromIdentifier();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = nullptr) override;
 };
 
 void DisplayOptionsTest::runIndexedTest(int32_t index, UBool exec, const char *&name, char *) {
