@@ -51,7 +51,7 @@ class LocaleMatcherTest : public IntlTest {
 public:
     LocaleMatcherTest() {}
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=NULL) override;
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=nullptr) override;
 
     void testEmpty();
     void testCopyErrorTo();
@@ -457,7 +457,7 @@ void LocaleMatcherTest::testResolvedLocale() {
 
 namespace {
 
-bool toInvariant(const UnicodeString &s, CharString &inv, ErrorCode &errorCode) {
+bool toInvariant(const UnicodeString &s, CharString &inv, IcuTestErrorCode &errorCode) {
     if (errorCode.isSuccess()) {
         inv.clear().appendInvariantChars(s, errorCode);
         return errorCode.isSuccess();
@@ -477,7 +477,7 @@ bool getSuffixAfterPrefix(const UnicodeString &s, int32_t limit,
 
 bool getInvariantSuffixAfterPrefix(const UnicodeString &s, int32_t limit,
                                    const UnicodeString &prefix, CharString &suffix,
-                                   ErrorCode &errorCode) {
+                                   IcuTestErrorCode &errorCode) {
     UnicodeString u_suffix;
     return getSuffixAfterPrefix(s, limit, prefix, u_suffix) &&
         toInvariant(u_suffix, suffix, errorCode);
@@ -578,7 +578,7 @@ UBool LocaleMatcherTest::dataDriven(const TestCase &test, IcuTestErrorCode &erro
     }
     if (!test.threshold.isEmpty()) {
         infoln("skipping test case on line %d with non-default threshold: not exposed via API",
-               (int)test.lineNr);
+               static_cast<int>(test.lineNr));
         return true;
         // int32_t threshold = Integer.valueOf(test.threshold);
         // builder.internalSetThresholdDistance(threshold);
@@ -651,7 +651,7 @@ void LocaleMatcherTest::testDataDriven() {
         return;
     }
     int32_t lineLength;
-    const UChar *p;
+    const char16_t *p;
     UnicodeString line;
     TestCase test;
     int32_t numPassed = 0;
@@ -660,20 +660,20 @@ void LocaleMatcherTest::testDataDriven() {
         line.setTo(false, p, lineLength);
         if (!readTestCase(line, test, errorCode)) {
             if (errorCode.errIfFailureAndReset(
-                    "test data syntax error on line %d", (int)test.lineNr)) {
+                    "test data syntax error on line %d", static_cast<int>(test.lineNr))) {
                 infoln(line);
             }
             continue;
         }
         UBool ok = dataDriven(test, errorCode);
-        if (errorCode.errIfFailureAndReset("test error on line %d", (int)test.lineNr)) {
+        if (errorCode.errIfFailureAndReset("test error on line %d", static_cast<int>(test.lineNr))) {
             infoln(line);
         } else if (!ok) {
-            infoln("test failure on line %d", (int)test.lineNr);
+            infoln("test failure on line %d", static_cast<int>(test.lineNr));
             infoln(line);
         } else {
             ++numPassed;
         }
     }
-    infoln("number of passing test cases: %d", (int)numPassed);
+    infoln("number of passing test cases: %d", static_cast<int>(numPassed));
 }
