@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1997-2003, International Business Machines Corporation and
@@ -22,7 +24,7 @@
  * NumberFormat.
  */
 class IntlTestNumberFormat: public IntlTest {
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );  
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = nullptr ) override;
 
 private:
 
@@ -79,7 +81,7 @@ public:
      **/
     static double randFraction()
     {
-        return (double)randLong() / (double)0xFFFFFFFF;
+        return static_cast<double>(randLong()) / static_cast<double>(0xFFFFFFFF);
     }
 
 };
