@@ -57,10 +57,10 @@ TimeZoneRegressionTest::failure(UErrorCode status, const char* msg)
 {
     if(U_FAILURE(status)) {
         errln(UnicodeString("FAIL: ") + msg + " failed, error " + u_errorName(status));
-        return TRUE;
+        return true;
     }
 
-    return FALSE;
+    return false;
 }
 
 /**
@@ -91,7 +91,7 @@ UDate TimeZoneRegressionTest::findTransitionBinary(const SimpleTimeZone& tz, UDa
     UBool startsInDST = tz.inDaylightTime(min, status);
     if (failure(status, "SimpleTimeZone::inDaylightTime")) return 0;
     if (tz.inDaylightTime(max, status) == startsInDST) {
-        logln((UnicodeString)"Error: inDaylightTime() != " + ((!startsInDST)?"TRUE":"FALSE"));
+        logln(UnicodeString("Error: inDaylightTime() != ") + (!startsInDST ? "true" : "false"));
         return 0;
     }
     if (failure(status, "SimpleTimeZone::inDaylightTime")) return 0;
@@ -116,7 +116,7 @@ UDate TimeZoneRegressionTest::findTransitionStepwise(const SimpleTimeZone& tz, U
             return min;
         }
         if (failure(status, "SimpleTimeZone::inDaylightTime")) return 0;
-        min += (UDate)24*60*60*1000; // one day
+        min += static_cast<UDate>(24) * 60 * 60 * 1000; // one day
     }
     return 0;
 }
@@ -153,7 +153,7 @@ void TimeZoneRegressionTest:: Test4073215()
     failure(status, "new GregorianCalendar");
     cal.adoptTimeZone(z.orphan());
 
-    SimpleDateFormat sdf((UnicodeString)"E d MMM yyyy G HH:mm", status); 
+    SimpleDateFormat sdf(UnicodeString("E d MMM yyyy G HH:mm"), status);
     if(U_FAILURE(status)) {
       dataerrln("Error creating date format %s", u_errorName(status));
       return;
@@ -166,7 +166,7 @@ void TimeZoneRegressionTest:: Test4073215()
     UBool indt = cal.getTimeZone().inDaylightTime(jan31 = cal.getTime(status), status);
     failure(status, "inDaylightTime or getTime call on Jan 31");
     if (indt) {
-        errln("Fail: Jan 31 inDaylightTime=TRUE, exp FALSE");
+        errln("Fail: Jan 31 inDaylightTime=true, exp false");
     }
     cal.set(1997, UCAL_MARCH, 1);
     indt = cal.getTimeZone().inDaylightTime(mar1 = cal.getTime(status), status);
@@ -175,13 +175,13 @@ void TimeZoneRegressionTest:: Test4073215()
         UnicodeString str;
         sdf.format(cal.getTime(status), str);
         failure(status, "getTime");
-        errln((UnicodeString)"Fail: " + str + " inDaylightTime=FALSE, exp TRUE");
+        errln(UnicodeString("Fail: ") + str + " inDaylightTime=false, exp true");
     }
     cal.set(1997, UCAL_MARCH, 31);
     indt = cal.getTimeZone().inDaylightTime(mar31 = cal.getTime(status), status);
     failure(status, "inDaylightTime or getTime call on Mar 31");
     if (indt) {
-        errln("Fail: Mar 31 inDaylightTime=TRUE, exp FALSE");
+        errln("Fail: Mar 31 inDaylightTime=true, exp false");
     }
 
     /*
@@ -242,8 +242,8 @@ void TimeZoneRegressionTest:: Test4084933() {
         1997, UCAL_APRIL, 6, UCAL_SUNDAY, (1*60*60*1000), status);
     int32_t offset8 = tz->getOffset(1,
         1997, UCAL_APRIL, 6, UCAL_SUNDAY, (1*60*60*1000)-1, status);
-    int32_t SToffset = (int32_t)(-8 * 60*60*1000L);
-    int32_t DToffset = (int32_t)(-7 * 60*60*1000L);
+    int32_t SToffset = static_cast<int32_t>(-8 * 60 * 60 * 1000L);
+    int32_t DToffset = static_cast<int32_t>(-7 * 60 * 60 * 1000L);
         
 #define ERR_IF_FAIL(x) if(x) { dataerrln("FAIL: TimeZone misbehaving - %s", #x); }
 
@@ -271,7 +271,7 @@ void TimeZoneRegressionTest:: Test4096952() {
     // {sfb} serialization not applicable
 /*
     UnicodeString ZONES [] = { UnicodeString("GMT"), UnicodeString("MET"), UnicodeString("IST") };
-    UBool pass = TRUE;
+    UBool pass = true;
     //try {
         for (int32_t i=0; i < ZONES.length; ++i) {
             TimeZone *zone = TimeZone::createTimeZone(ZONES[i]);
@@ -316,7 +316,7 @@ void TimeZoneRegressionTest:: Test4096952() {
  */
 void TimeZoneRegressionTest:: Test4109314() {
     UErrorCode status = U_ZERO_ERROR;
-    GregorianCalendar *testCal = (GregorianCalendar*)Calendar::createInstance(status); 
+    GregorianCalendar *testCal = dynamic_cast<GregorianCalendar*>(Calendar::createInstance(status));
     if(U_FAILURE(status)) {
       dataerrln("Error creating calendar %s", u_errorName(status));
       delete testCal;
@@ -334,7 +334,7 @@ void TimeZoneRegressionTest:: Test4109314() {
         CalendarRegressionTest::makeDate(98,UCAL_OCTOBER,24,22,0), 
         CalendarRegressionTest::makeDate(98,UCAL_OCTOBER,25,6,0)
     };
-    UBool pass = TRUE;
+    UBool pass = true;
     for (int32_t i = 0; i < 4; i+=2) {
         //testCal->setTimeZone((TimeZone) testData[i]);
         testCal->setTimeZone(*PST);
@@ -343,7 +343,7 @@ void TimeZoneRegressionTest:: Test4109314() {
         while(testCal->getTime(status) < end) { 
             testCal->setTime(t, status);
             if ( ! checkCalendar314(testCal, PST))
-                pass = FALSE;
+                pass = false;
             t += 60*60*1000.0;
         } 
     }
@@ -389,21 +389,21 @@ TimeZoneRegressionTest::checkCalendar314(GregorianCalendar *testCal, TimeZone *t
         dow = UCAL_SUNDAY + ((dow - UCAL_SUNDAY + 1) % 7);
     }
 
-    tzOffset = testTZ->getOffset((uint8_t)testCal->get(UCAL_ERA, status), 
+    tzOffset = testTZ->getOffset(static_cast<uint8_t>(testCal->get(UCAL_ERA, status)),
                                 testCal->get(UCAL_YEAR, status), 
                                 testCal->get(UCAL_MONTH, status), 
                                 date, 
-                                (uint8_t)dow, 
-                                (int32_t)millis,
+                                static_cast<uint8_t>(dow),
+                                static_cast<int32_t>(millis),
                                 status); 
     tzRawOffset = testTZ->getRawOffset(); 
-    tzOffsetFloat = (float)tzOffset/(float)3600000; 
-    tzRawOffsetFloat = (float)tzRawOffset/(float)3600000; 
+    tzOffsetFloat = static_cast<float>(tzOffset) / static_cast<float>(3600000);
+    tzRawOffsetFloat = static_cast<float>(tzRawOffset) / static_cast<float>(3600000);
 
     UDate testDate = testCal->getTime(status); 
 
     UBool inDaylightTime = testTZ->inDaylightTime(testDate, status); 
-    LocalPointer<SimpleDateFormat> sdf(new SimpleDateFormat((UnicodeString) "MM/dd/yyyy HH:mm", status),
+    LocalPointer<SimpleDateFormat> sdf(new SimpleDateFormat(UnicodeString("MM/dd/yyyy HH:mm"), status),
                                        status);
     if (U_FAILURE(status)) {
         errln("Error creating SimpleDateFormat %s", u_errorName(status));
@@ -430,7 +430,7 @@ TimeZoneRegressionTest::checkCalendar314(GregorianCalendar *testCal, TimeZone *t
     output = testTZ->getID(output) + " " + sdf->format(testDate, output, pos) +
         " Offset(" + tzOffsetFloat + ")" +
         " RawOffset(" + tzRawOffsetFloat + ")" + 
-        " " + millis/(float)3600000 + " " +
+        " " + millis / static_cast<float>(3600000) + " " +
         inDaylightTimeString; 
 
     if (passed) 
@@ -490,7 +490,7 @@ void TimeZoneRegressionTest:: Test4126678()
         cal->get(UCAL_DST_OFFSET, status);
 
     failure(status, "cal->get");
-    int32_t offset = tz->getOffset((uint8_t)era, year, month, day, (uint8_t)dayOfWeek, millis, status);
+    int32_t offset = tz->getOffset(static_cast<uint8_t>(era), year, month, day, static_cast<uint8_t>(dayOfWeek), millis, status);
     int32_t raw_offset = tz->getRawOffset();
 
     if (offset == raw_offset)
@@ -526,7 +526,7 @@ void TimeZoneRegressionTest:: Test4151406() {
             if (count> max) 
                 max = count;
             if (count > 0) {
-                logln(hname + ' ' + (UnicodeString)count + (UnicodeString)" e.g. " + *ids->snext(ec));
+                logln(hname + ' ' + UnicodeString(count) + UnicodeString(" e.g. ") + *ids->snext(ec));
             } else {
                 logln(hname + ' ' + count);
             }
@@ -549,7 +549,7 @@ void TimeZoneRegressionTest:: Test4151429() {
     //try {
         /*TimeZone *tz = TimeZone::createTimeZone("GMT");
         UnicodeString name;
-        tz->getDisplayName(TRUE, TimeZone::LONG,
+        tz->getDisplayName(true, TimeZone::LONG,
                                         Locale.getDefault(), name);
         errln("IllegalArgumentException not thrown by TimeZone::getDisplayName()");*/
     //} catch(IllegalArgumentException e) {}
@@ -674,8 +674,8 @@ void TimeZoneRegressionTest:: Test4154542()
         // {sfb} need to look into ctor problems! (UErrorCode vs. dst signature confusion)
         status = U_ZERO_ERROR;
             SimpleTimeZone *temp = new SimpleTimeZone(0, "Z",
-                    (int8_t)month, (int8_t)day, (int8_t)dayOfWeek, time,
-                    (int8_t)GOOD_MONTH, (int8_t)GOOD_DAY, (int8_t)GOOD_DAY_OF_WEEK, 
+                    static_cast<int8_t>(month), static_cast<int8_t>(day), static_cast<int8_t>(dayOfWeek), time,
+                    static_cast<int8_t>(GOOD_MONTH), static_cast<int8_t>(GOOD_DAY), static_cast<int8_t>(GOOD_DAY_OF_WEEK),
                     GOOD_TIME,status);
         if (temp == nullptr) {
             errln("Fail: failed to create SimpleTimeZone %s", u_errorName(status));
@@ -696,9 +696,9 @@ void TimeZoneRegressionTest:: Test4154542()
         //try {
         status = U_ZERO_ERROR;
             temp = new SimpleTimeZone(0, "Z",
-                    (int8_t)GOOD_MONTH, (int8_t)GOOD_DAY, (int8_t)GOOD_DAY_OF_WEEK, 
+                    static_cast<int8_t>(GOOD_MONTH), static_cast<int8_t>(GOOD_DAY), static_cast<int8_t>(GOOD_DAY_OF_WEEK),
                     GOOD_TIME,
-                    (int8_t)month, (int8_t)day, (int8_t)dayOfWeek, time,status);
+                    static_cast<int8_t>(month), static_cast<int8_t>(day), static_cast<int8_t>(dayOfWeek), time, status);
         if (temp == nullptr) {
             errln("Fail: failed to create SimpleTimeZone %s", u_errorName(status));
             return;
@@ -830,8 +830,8 @@ TimeZoneRegressionTest::Test4154650()
         //IllegalArgumentException e = null;
         //try {
             /*int32_t offset = */
-        tz->getOffset((uint8_t)DATA[i+1], DATA[i+2], DATA[i+3],
-                      DATA[i+4], (uint8_t)DATA[i+5], DATA[i+6], status); 
+        tz->getOffset(static_cast<uint8_t>(DATA[i + 1]), DATA[i + 2], DATA[i + 3],
+                      DATA[i + 4], static_cast<uint8_t>(DATA[i + 5]), DATA[i + 6], status);
         //} catch (IllegalArgumentException ex) {
         //   e = ex;
         //}
@@ -882,8 +882,7 @@ TimeZoneRegressionTest::Test4162593()
      * Starting time
      * Transition expected between start+1H and start+2H
      */
-    TimeZone *DATA_TZ [] = {
-      0, 0, 0 };
+    TimeZone* DATA_TZ[] = {nullptr, nullptr, nullptr};
 
     int32_t DATA_INT [] [5] = {
         // These years must be AFTER the Gregorian cutover
@@ -911,7 +910,7 @@ TimeZoneRegressionTest::Test4162593()
     }
 
     for(int32_t j = 0; j < 3; j++) {
-        TimeZone *tz = (TimeZone*)DATA_TZ[j];
+        TimeZone *tz = DATA_TZ[j];
         TimeZone::setDefault(*tz);
         fmt->setTimeZone(*tz);
 
@@ -979,21 +978,21 @@ void TimeZoneRegressionTest::Test4176686() {
     UnicodeString a,b,c,d,e,f,g,h,i,j,k,l;
     UnicodeString DATA[] = {
         "z1.getDisplayName(false, SHORT)/std zone",
-        z1.getDisplayName(FALSE, TimeZone::SHORT, a), "GMT+1:30",
+        z1.getDisplayName(false, TimeZone::SHORT, a), "GMT+1:30",
         "z1.getDisplayName(false, LONG)/std zone",
-        z1.getDisplayName(FALSE, TimeZone::LONG, b), "GMT+01:30",
+        z1.getDisplayName(false, TimeZone::LONG, b), "GMT+01:30",
         "z1.getDisplayName(true, SHORT)/std zone",
-        z1.getDisplayName(TRUE, TimeZone::SHORT, c), "GMT+1:30",
+        z1.getDisplayName(true, TimeZone::SHORT, c), "GMT+1:30",
         "z1.getDisplayName(true, LONG)/std zone",
-        z1.getDisplayName(TRUE, TimeZone::LONG, d ), "GMT+01:30",
+        z1.getDisplayName(true, TimeZone::LONG, d ), "GMT+01:30",
         "z2.getDisplayName(false, SHORT)/dst zone",
-        z2.getDisplayName(FALSE, TimeZone::SHORT, e), "GMT+1:30",
+        z2.getDisplayName(false, TimeZone::SHORT, e), "GMT+1:30",
         "z2.getDisplayName(false, LONG)/dst zone",
-        z2.getDisplayName(FALSE, TimeZone::LONG, f ), "GMT+01:30",
+        z2.getDisplayName(false, TimeZone::LONG, f ), "GMT+01:30",
         "z2.getDisplayName(true, SHORT)/dst zone",
-        z2.getDisplayName(TRUE, TimeZone::SHORT, g), "GMT+2:15",
+        z2.getDisplayName(true, TimeZone::SHORT, g), "GMT+2:15",
         "z2.getDisplayName(true, LONG)/dst zone",
-        z2.getDisplayName(TRUE, TimeZone::LONG, h ), "GMT+02:15",
+        z2.getDisplayName(true, TimeZone::LONG, h ), "GMT+02:15",
         "DateFormat.format(std)/std zone", fmt1.format(std, i), "GMT+1:30",
         "DateFormat.format(dst)/std zone", fmt1.format(dst, j), "GMT+1:30",
         "DateFormat.format(std)/dst zone", fmt2.format(std, k), "GMT+1:30",
@@ -1078,11 +1077,11 @@ void TimeZoneRegressionTest::TestJ449() {
             fmt.setCalendar(calWithout);
             fmt.format(date, strWithout);
             if (strWith == strWithout) {
-                logln((UnicodeString)"Ok: " + idWithLocaleData + " -> " +
+                logln(UnicodeString("Ok: ") + idWithLocaleData + " -> " +
                       strWith + "; " + idWithoutLocaleData + " -> " +
                       strWithout);
             } else {
-                errln((UnicodeString)"FAIL: " + idWithLocaleData + " -> " +
+                errln(UnicodeString("FAIL: ") + idWithLocaleData + " -> " +
                       strWith + "; " + idWithoutLocaleData + " -> " +
                       strWithout);
             }
@@ -1159,7 +1158,7 @@ TimeZoneRegressionTest::TestJDK12API()
  * SimpleTimeZone allows invalid DOM values.
  */
 void TimeZoneRegressionTest::Test4184229() {
-    SimpleTimeZone* zone = NULL;
+    SimpleTimeZone* zone = nullptr;
     UErrorCode status = U_ZERO_ERROR;
     zone = new SimpleTimeZone(0, "A", 0, -1, 0, 0, 0, 0, 0, 0, status);
     if(U_SUCCESS(status)){
@@ -1238,7 +1237,7 @@ void TimeZoneRegressionTest::TestNegativeDaylightSaving() {
     failure(status, "SimpleTimeZone constructor");
 
     if (save != stzDublin.getDSTSavings()) {
-        errln((UnicodeString)"FAIL: DST saving is not " + save);
+        errln(UnicodeString("FAIL: DST saving is not ") + save);
     }
 
     GregorianCalendar cal(* TimeZone::getGMT(), status);
@@ -1257,10 +1256,10 @@ void TimeZoneRegressionTest::TestNegativeDaylightSaving() {
     }
     failure(status, "inDaylightTime() - Jan 15");
 
-    stzDublin.getOffset(testDate, FALSE, rawOffset, dstOffset, status);
+    stzDublin.getOffset(testDate, false, rawOffset, dstOffset, status);
     failure(status, "getOffset() - Jan 15");
     if (rawOffset != stdOff || dstOffset != save) {
-        errln((UnicodeString)"FAIL: Expected [stdoff=" + stdOff + ",save=" + save
+        errln(UnicodeString("FAIL: Expected [stdoff=") + stdOff + ",save=" + save
             + "] on the test date (Jan 15), actual[stdoff=" + rawOffset
             + ",save=" + dstOffset + "]");
     }
@@ -1274,10 +1273,10 @@ void TimeZoneRegressionTest::TestNegativeDaylightSaving() {
     }
     failure(status, "inDaylightTime() - Jul 15");
 
-    stzDublin.getOffset(testDate, FALSE, rawOffset, dstOffset, status);
+    stzDublin.getOffset(testDate, false, rawOffset, dstOffset, status);
     failure(status, "getOffset() - Jul 15");
     if (rawOffset != stdOff || dstOffset != 0) {
-        errln((UnicodeString)"FAIL: Expected [stdoff=" + stdOff + ",save=" + 0
+        errln(UnicodeString("FAIL: Expected [stdoff=") + stdOff + ",save=" + 0
             + "] on the test date (Jul 15), actual[stdoff=" + rawOffset
             + ",save=" + dstOffset + "]");
     }
