@@ -92,6 +92,7 @@ public:
     void TestUnicodeStringImplementsAppendable();
     void TestSizeofUnicodeString();
     void TestMoveSwap();
+    void TestLargeMemory();
 
     void TestUInt16Pointers();
     void TestWCharPointers();
@@ -100,6 +101,7 @@ public:
     void TestLargeAppend();
     void TestU16StringView();
     void TestWStringView();
+    void TestRange();
 };
 
 #endif
