@@ -16,7 +16,6 @@
 #include "unicode/utypes.h"
 #include "unicode/uclean.h"
 #include "unicode/uchar.h"
-#include "unicode/unistr.h"
 #include "unicode/uscript.h"
 #include "unicode/putil.h"
 #include "unicode/ctest.h"
@@ -28,13 +27,10 @@
 #include "layout/ParagraphLayout.h"
 #include "layout/RunArrays.h"
 
-#include "PortableFontInstance.h"
 #include "SimpleFontInstance.h"
 
-#include "letsutil.h"
 #include "letest.h"
 
-#include "xmlparser.h"
 #include "putilimp.h" // for uprv_getUTCtime()
 
 #include <stdlib.h>
@@ -42,17 +38,8 @@
 
 U_NAMESPACE_USE
 
-#define CH_COMMA 0x002C
-
 U_CDECL_BEGIN
 
-static void U_CALLCONV ScriptTest()
-{
-    if ((int)scriptCodeCount != (int)USCRIPT_CODE_LIMIT) {
-        log_err("ScriptCodes::scriptCodeCount = %d, but UScriptCode::USCRIPT_CODE_LIMIT = %d\n", scriptCodeCount, USCRIPT_CODE_LIMIT);
-    }
-}
-
 static void U_CALLCONV ParamTest()
 {
     LEErrorCode status = LE_NO_ERROR;
@@ -327,415 +314,6 @@ bail:
 }
 U_CDECL_END
 
-le_bool compareResults(const char *testID, TestResult *expected, TestResult *actual)
-{
-    /* NOTE: we'll stop on the first failure 'cause once there's one error, it may cascade... */
-    if (actual->glyphCount != expected->glyphCount) {
-        log_err("Test %s: incorrect glyph count: expected %d, got %d\n",
-            testID, expected->glyphCount, actual->glyphCount);
-        return false;
-    }
-
-    le_int32 i;
-
-    for (i = 0; i < actual->glyphCount; i += 1) {
-        if (actual->glyphs[i] != expected->glyphs[i]) {
-            log_err("Test %s: incorrect id for glyph %d: expected %4X, got %4X\n",
-                testID, i, expected->glyphs[i], actual->glyphs[i]);
-            return false;
-        }
-    }
-
-    for (i = 0; i < actual->glyphCount; i += 1) {
-        if (actual->indices[i] != expected->indices[i]) {
-            log_err("Test %s: incorrect index for glyph %d: expected %8X, got %8X\n",
-                testID, i, expected->indices[i], actual->indices[i]);
-            return false;
-        }
-    }
-
-    for (i = 0; i <= actual->glyphCount; i += 1) {
-        double xError = uprv_fabs(actual->positions[i * 2] - expected->positions[i * 2]);
-
-        if (xError > 0.0001) {
-            log_err("Test %s: incorrect x position for glyph %d: expected %f, got %f\n",
-                testID, i, expected->positions[i * 2], actual->positions[i * 2]);
-            return false;
-        }
-
-        double yError = uprv_fabs(actual->positions[i * 2 + 1] - expected->positions[i * 2 + 1]);
-
-        if (yError < 0) {
-            yError = -yError;
-        }
-
-        if (yError > 0.0001) {
-            log_err("Test %s: incorrect y position for glyph %d: expected %f, got %f\n",
-                testID, i, expected->positions[i * 2 + 1], actual->positions[i * 2 + 1]);
-            return false;
-        }
-    }
-
-    return true;
-}
-
-static void checkFontVersion(PortableFontInstance *fontInstance, const char *testVersionString,
-                             le_uint32 testChecksum, const char *testID)
-{
-    le_uint32 fontChecksum = fontInstance->getFontChecksum();
-
-    if (fontChecksum != testChecksum) {
-        const char *fontVersionString = fontInstance->getNameString(NAME_VERSION_STRING,
-            PLATFORM_MACINTOSH, MACINTOSH_ROMAN, MACINTOSH_ENGLISH);
-        const LEUnicode *uFontVersionString = nullptr;
-
-            // The standard recommends that the Macintosh Roman/English name string be present, but
-            // if it's not, try the Microsoft Unicode/English string.
-            if (fontVersionString == nullptr) {
-                uFontVersionString = fontInstance->getUnicodeNameString(NAME_VERSION_STRING,
-                    PLATFORM_MICROSOFT, MICROSOFT_UNICODE_BMP, MICROSOFT_ENGLISH);
-            }
-
-        log_info("Test %s: this may not be the same font used to generate the test data.\n", testID);
-
-        if (uFontVersionString != nullptr) {
-            log_info("Your font's version string is \"%S\"\n", uFontVersionString);
-            fontInstance->deleteNameString(uFontVersionString);
-        } else {
-            log_info("Your font's version string is \"%s\"\n", fontVersionString);
-            fontInstance->deleteNameString(fontVersionString);
-        }
-
-        log_info("The expected version string is \"%s\"\n", testVersionString);
-        log_info("If you see errors, they may be due to the version of the font you're using.\n");
-    }
-}
-
-/* Returns the path to icu/source/test/testdata/ */
-const char *getSourceTestData() {
-    const char *srcDataDir = nullptr;
-#ifdef U_TOPSRCDIR
-    srcDataDir = U_TOPSRCDIR U_FILE_SEP_STRING "test" U_FILE_SEP_STRING "testdata" U_FILE_SEP_STRING;
-#else
-    srcDataDir = ".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING "test" U_FILE_SEP_STRING "testdata" U_FILE_SEP_STRING;
-    FILE *f = fopen(".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING "test" U_FILE_SEP_STRING "testdata" U_FILE_SEP_STRING "rbbitst.txt", "r");
-
-    if (f != nullptr) {
-        /* We're in icu/source/test/letest/ */
-        fclose(f);
-    } else {
-        /* We're in icu/source/test/letest/(Debug|Release) */
-        srcDataDir =  ".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING "test" 
-                      U_FILE_SEP_STRING "testdata" U_FILE_SEP_STRING;
-    }
-#endif
-
-    return srcDataDir;
-}
-
-const char *getPath(char buffer[2048], const char *filename) {
-    const char *testDataDirectory = getSourceTestData();
-
-    strcpy(buffer, testDataDirectory);
-    strcat(buffer, filename);
-
-    return buffer;
-}
-
-le_uint32 *getHexArray(const UnicodeString &numbers, int32_t &arraySize)
-{
-    int32_t offset = -1;
-
-    arraySize = 1;
-    while((offset = numbers.indexOf(CH_COMMA, offset + 1)) >= 0) {
-        arraySize += 1;
-    }
-
-    le_uint32 *array = NEW_ARRAY(le_uint32, arraySize);
-    char number[16];
-    le_int32 count = 0;
-    le_int32 start = 0, end = 0;
-    le_int32 len = 0;
-
-    // trim leading whitespace
-    while(u_isUWhiteSpace(numbers[start])) {
-        start += 1;
-    }
-
-    while((end = numbers.indexOf(CH_COMMA, start)) >= 0) {
-        len = numbers.extract(start, end - start, number, ARRAY_SIZE(number), US_INV);
-        number[len] = '\0';
-        start = end + 1;
-
-        sscanf(number, "%x", &array[count++]);
-
-        // trim whitespace following the comma
-        while(u_isUWhiteSpace(numbers[start])) {
-            start += 1;
-        }
-    }
-
-    // trim trailing whitespace
-    end = numbers.length();
-    while(u_isUWhiteSpace(numbers[end - 1])) {
-        end -= 1;
-    }
-
-    len = numbers.extract(start, end - start, number, ARRAY_SIZE(number), US_INV);
-    number[len] = '\0';
-    sscanf(number, "%x", &array[count]);
-
-    return array;
-}
-
-float *getFloatArray(const UnicodeString &numbers, int32_t &arraySize)
-{
-    int32_t offset = -1;
-
-    arraySize = 1;
-    while((offset = numbers.indexOf(CH_COMMA, offset + 1)) >= 0) {
-        arraySize += 1;
-    }
-
-    float *array = NEW_ARRAY(float, arraySize);
-    char number[32];
-    le_int32 count = 0;
-    le_int32 start = 0, end = 0;
-    le_int32 len = 0;
-
-    // trim leading whitespace
-    while(u_isUWhiteSpace(numbers[start])) {
-        start += 1;
-    }
-
-    while((end = numbers.indexOf(CH_COMMA, start)) >= 0) {
-        len = numbers.extract(start, end - start, number, ARRAY_SIZE(number), US_INV);
-        number[len] = '\0';
-        start = end + 1;
-
-        sscanf(number, "%f", &array[count++]);
-
-        // trim whiteapce following the comma
-        while(u_isUWhiteSpace(numbers[start])) {
-            start += 1;
-        }
-    }
-
-    while(u_isUWhiteSpace(numbers[start])) {
-        start += 1;
-    }
-
-    // trim trailing whitespace
-    end = numbers.length();
-    while(u_isUWhiteSpace(numbers[end - 1])) {
-        end -= 1;
-    }
-
-    len = numbers.extract(start, end - start, number, ARRAY_SIZE(number), US_INV);
-    number[len] = '\0';
-    sscanf(number, "%f", &array[count]);
-
-    return array;
-}
-
-LEFontInstance *openFont(const char *fontName, const char *checksum, const char *version, const char *testID)
-{
-    char path[2048];
-    PortableFontInstance *font;
-    LEErrorCode fontStatus = LE_NO_ERROR;
-
-
-    font = new PortableFontInstance(getPath(path, fontName), 12, fontStatus);
-
-    if (LE_FAILURE(fontStatus)) {
-        log_info("Test %s: can't open font %s - test skipped.\n", testID, fontName);
-        delete font;
-        return nullptr;
-    } else {
-        le_uint32 cksum = 0;
-
-        sscanf(checksum, "%x", &cksum);
-
-        checkFontVersion(font, version, cksum, testID);
-    }
-
-    return font;
-}
-
-U_CDECL_BEGIN
-static void U_CALLCONV DataDrivenTest()
-{
-#if !UCONFIG_NO_REGULAR_EXPRESSIONS
-    UErrorCode status = U_ZERO_ERROR;
-    char path[2048];
-    const char *testFilePath = getPath(path, "letest.xml");
-
-    UXMLParser  *parser = UXMLParser::createParser(status);
-    UXMLElement *root   = parser->parseFile(testFilePath, status);
-
-    if (root == nullptr) {
-        log_err("Could not open the test data file: %s\n", testFilePath);
-        delete parser;
-        return;
-    }
-
-    UnicodeString test_case        = UNICODE_STRING_SIMPLE("test-case");
-    UnicodeString test_text        = UNICODE_STRING_SIMPLE("test-text");
-    UnicodeString test_font        = UNICODE_STRING_SIMPLE("test-font");
-    UnicodeString result_glyphs    = UNICODE_STRING_SIMPLE("result-glyphs");
-    UnicodeString result_indices   = UNICODE_STRING_SIMPLE("result-indices");
-    UnicodeString result_positions = UNICODE_STRING_SIMPLE("result-positions");
-
-    // test-case attributes
-    UnicodeString id_attr     = UNICODE_STRING_SIMPLE("id");
-    UnicodeString script_attr = UNICODE_STRING_SIMPLE("script");
-    UnicodeString lang_attr   = UNICODE_STRING_SIMPLE("lang");
-
-    // test-font attributes
-    UnicodeString name_attr   = UNICODE_STRING_SIMPLE("name");
-    UnicodeString ver_attr    = UNICODE_STRING_SIMPLE("version");
-    UnicodeString cksum_attr  = UNICODE_STRING_SIMPLE("checksum");
-
-    const UXMLElement *testCase;
-    int32_t tc = 0;
-
-    while((testCase = root->nextChildElement(tc)) != nullptr) {
-        if (testCase->getTagName().compare(test_case) == 0) {
-            char *id = getCString(testCase->getAttribute(id_attr));
-            char *script = getCString(testCase->getAttribute(script_attr));
-            char *lang   = getCString(testCase->getAttribute(lang_attr));
-            LEFontInstance *font = nullptr;
-            const UXMLElement *element;
-            int32_t ec = 0;
-            int32_t charCount = 0;
-            int32_t typoFlags = 3; // kerning + ligatures...
-            UScriptCode scriptCode;
-            le_int32 languageCode = -1;
-            UnicodeString text, glyphs, indices, positions;
-            int32_t glyphCount = 0, indexCount = 0, positionCount = 0;
-            TestResult expected = {0, nullptr, nullptr, nullptr};
-            TestResult actual   = {0, nullptr, nullptr, nullptr};
-            LEErrorCode success = LE_NO_ERROR;
-            LayoutEngine *engine = nullptr;
-
-            uscript_getCode(script, &scriptCode, 1, &status);
-            if (LE_FAILURE(status)) {
-                log_err("invalid script name: %s.\n", script);
-                goto free_c_strings;
-            }
-
-            if (lang != nullptr) {
-                languageCode = getLanguageCode(lang);
-
-                if (languageCode < 0) {
-                    log_err("invalid language name: %s.\n", lang);
-                    goto free_c_strings;
-                }
-            }
-
-            while((element = testCase->nextChildElement(ec)) != nullptr) {
-                UnicodeString tag = element->getTagName();
-
-                // TODO: make sure that each element is only used once.
-                if (tag.compare(test_font) == 0) {
-                    char *fontName  = getCString(element->getAttribute(name_attr));
-                    char *fontVer   = getCString(element->getAttribute(ver_attr));
-                    char *fontCksum = getCString(element->getAttribute(cksum_attr));
-
-                    font = openFont(fontName, fontCksum, fontVer, id);
-                    freeCString(fontCksum);
-                    freeCString(fontVer);
-                    freeCString(fontName);
-
-                    if (font == nullptr) {
-                        // warning message already displayed...
-                        goto free_c_strings;
-                    }
-                } else if (tag.compare(test_text) == 0) {
-                    text = element->getText(true);
-                    charCount = text.length();
-                } else if (tag.compare(result_glyphs) == 0) {
-                    glyphs = element->getText(true);
-                } else if (tag.compare(result_indices) == 0) {
-                    indices = element->getText(true);
-                } else if (tag.compare(result_positions) == 0) {
-                    positions = element->getText(true);
-                } else {
-                    // an unknown tag...
-                    char *cTag = getCString(&tag);
-
-                    log_info("Test %s: unknown element with tag \"%s\"\n", id, cTag);
-                    freeCString(cTag);
-                }
-            }
-
-            // TODO: make sure that the font, test-text, result-glyphs, result-indices and result-positions
-            // have all been provided
-            if (font == nullptr) {
-                LEErrorCode fontStatus = LE_NO_ERROR;
-
-                font = new SimpleFontInstance(12, fontStatus);
-                typoFlags |= 0x80000000L;  // use CharSubstitutionFilter...
-            }
-
-            expected.glyphs    = (LEGlyphID *) getHexArray(glyphs, glyphCount);
-            expected.indices   = (le_int32 *)  getHexArray(indices, indexCount);
-            expected.positions = getFloatArray(positions, positionCount);
-
-            expected.glyphCount = glyphCount;
-
-            if (glyphCount < charCount || indexCount != glyphCount || positionCount < glyphCount * 2 + 2) {
-                log_err("Test %s: inconsistent input data: charCount = %d, glyphCount = %d, indexCount = %d, positionCount = %d\n",
-                    id, charCount, glyphCount, indexCount, positionCount);
-                goto free_expected;
-            };
-
-            engine = LayoutEngine::layoutEngineFactory(font, scriptCode, languageCode, typoFlags, success);
-
-            if (LE_FAILURE(success)) {
-                log_err("Test %s: could not create a LayoutEngine.\n", id);
-                goto free_expected;
-            }
-
-            actual.glyphCount = engine->layoutChars(text.getBuffer(), 0, charCount, charCount, getRTL(text), 0, 0, success);
-
-            actual.glyphs    = NEW_ARRAY(LEGlyphID, actual.glyphCount);
-            actual.indices   = NEW_ARRAY(le_int32, actual.glyphCount);
-            actual.positions = NEW_ARRAY(float, actual.glyphCount * 2 + 2);
-
-            engine->getGlyphs(actual.glyphs, success);
-            engine->getCharIndices(actual.indices, success);
-            engine->getGlyphPositions(actual.positions, success);
-
-            compareResults(id, &expected, &actual);
-
-            DELETE_ARRAY(actual.positions);
-            DELETE_ARRAY(actual.indices);
-            DELETE_ARRAY(actual.glyphs);
-
-            delete engine;
-
-            log_verbose("OK - %4d glyphs: %s\n", actual.glyphCount, id);
-free_expected:
-            DELETE_ARRAY(expected.positions);
-            DELETE_ARRAY(expected.indices);
-            DELETE_ARRAY(expected.glyphs);
-
-            delete font;
-
-free_c_strings:
-            freeCString(lang);
-            freeCString(script);
-            freeCString(id);
-        }
-    }
-
-    delete root;
-    delete parser;
-#endif
-}
-U_CDECL_END
-
 U_CDECL_BEGIN
 /*
  * From ticket:5923:
@@ -968,16 +546,10 @@ U_CDECL_END
 
 static void addAllTests(TestNode **root)
 {
-    addTest(root, &ScriptTest,      "api/ScriptTest");
     addTest(root, &ParamTest,       "api/ParameterTest");
     addTest(root, &FactoryTest,     "api/FactoryTest");
     addTest(root, &AccessTest,      "layout/AccessTest");
-    addTest(root, &DataDrivenTest,  "layout/DataDrivenTest");
     addTest(root, &GlyphToCharTest, "paragraph/GlyphToCharTest");
-
-#ifndef USING_ICULEHB
-    addCTests(root);
-#endif
 }
 
 /* returns the path to icu/source/data/out */
@@ -1122,12 +694,12 @@ int main(int argc, char* argv[])
     u_cleanup();
 
     endTime = uprv_getUTCtime();
-    diffTime = (int32_t)(endTime - startTime);
+    diffTime = static_cast<int32_t>(endTime - startTime);
     printf("Elapsed Time: %02d:%02d:%02d.%03d\n",
-        (int)((diffTime%U_MILLIS_PER_DAY)/U_MILLIS_PER_HOUR),
-        (int)((diffTime%U_MILLIS_PER_HOUR)/U_MILLIS_PER_MINUTE),
-        (int)((diffTime%U_MILLIS_PER_MINUTE)/U_MILLIS_PER_SECOND),
-        (int)(diffTime%U_MILLIS_PER_SECOND));
+        (diffTime % U_MILLIS_PER_DAY) / U_MILLIS_PER_HOUR,
+        (diffTime % U_MILLIS_PER_HOUR) / U_MILLIS_PER_MINUTE,
+        (diffTime % U_MILLIS_PER_MINUTE) / U_MILLIS_PER_SECOND,
+        diffTime % U_MILLIS_PER_SECOND);
 
     return nerrors;
 }
