@@ -30,7 +30,7 @@ typedef std::wstring stlstring;
 //#define LOOPS 10
 #define catenate_STRLEN 2
 
-const UChar uTESTCHAR1 =  'a';
+const char16_t uTESTCHAR1 =  'a';
 const wchar_t wTESTCHAR1 = 'a';
 const UnicodeString uEMPTY;
 const stlstring sEMPTY;
@@ -60,7 +60,7 @@ struct  WLine {
 
 enum FnType { Fn_ICU, Fn_STD };
 typedef FnType FnType;
-typedef void (*ICUStringPerfFn)(const UChar* src,int32_t srcLen, UnicodeString s0);
+typedef void (*ICUStringPerfFn)(const char16_t* src,int32_t srcLen, UnicodeString s0);
 typedef void (*StdStringPerfFn)(const wchar_t* src,int32_t srcLen, stlstring s0);
 
 
@@ -68,10 +68,10 @@ class StringPerfFunction : public UPerfFunction
 {
 public:
 
-    virtual long getEventsPerIteration(){
+    long getEventsPerIteration() override {
         int loops = LOOPS;
-        if (catICU) { delete catICU;}
-        if (catStd) { delete catStd;}
+        delete catICU;
+        delete catStd;
 
         if (bCatenatePrealloc) {
 
@@ -90,7 +90,7 @@ public:
         return -1;
     }
 
-    virtual void call(UErrorCode* status)
+    void call(UErrorCode* status) override
     {
         if(line_mode_==true){
             if(uselen_){
@@ -127,7 +127,7 @@ public:
         }
     }
 
-    virtual long getOperationsPerIteration()
+    long getOperationsPerIteration() override
     {
         if(line_mode_==true){
             return numLines_;
@@ -141,13 +141,13 @@ public:
 
         fn1_ = func;
         lines_=srcLines;
-        wlines_=NULL;
+        wlines_=nullptr;
         numLines_=srcNumLines;
         uselen_=uselen;
         line_mode_=true;
-        src_ = NULL;
+        src_ = nullptr;
         srcLen_ = 0;
-        wsrc_ = NULL;
+        wsrc_ = nullptr;
         wsrcLen_ = 0;
         fnType_ = Fn_ICU;
 
@@ -155,9 +155,9 @@ public:
         for(int32_t i=0; i<numLines_; i++) {
             uS0_[i]=UnicodeString(lines_[i].name, lines_[i].len);
         }
-        sS0_=NULL;
-        ubulk_=NULL;
-        sbulk_=NULL;
+        sS0_=nullptr;
+        ubulk_=nullptr;
+        sbulk_=nullptr;
     }
 
     StringPerfFunction(StdStringPerfFn func, ULine* srcLines, int32_t srcNumLines, UBool uselen)
@@ -165,63 +165,63 @@ public:
 
         fn2_ = func;
         lines_=srcLines;
-        wlines_=NULL;
+        wlines_=nullptr;
         numLines_=srcNumLines;
         uselen_=uselen;
         line_mode_=true;
-        src_ = NULL;
+        src_ = nullptr;
         srcLen_ = 0;
-        wsrc_ = NULL;
+        wsrc_ = nullptr;
         wsrcLen_ = 0;
         fnType_ = Fn_STD;
 
-        uS0_=NULL;
-        ubulk_=NULL;
-        sbulk_=NULL;
+        uS0_=nullptr;
+        ubulk_=nullptr;
+        sbulk_=nullptr;
 
         //fillin wlines_[], sS0_[]
         prepareLinesForStd();
     }
 
-    StringPerfFunction(ICUStringPerfFn func, UChar* source, int32_t sourceLen, UBool uselen)
+    StringPerfFunction(ICUStringPerfFn func, char16_t* source, int32_t sourceLen, UBool uselen)
     {
 
         fn1_ = func;
-        lines_=NULL;
-        wlines_=NULL;
+        lines_=nullptr;
+        wlines_=nullptr;
         numLines_=0;
         uselen_=uselen;
         line_mode_=false;
-        src_ = new UChar[sourceLen];
+        src_ = new char16_t[sourceLen];
         memcpy(src_, source, sourceLen * U_SIZEOF_UCHAR);
         srcLen_ = sourceLen;
-        wsrc_ = NULL;
+        wsrc_ = nullptr;
         wsrcLen_ = 0;
         fnType_ = Fn_ICU;
 
-        uS0_=NULL;
-        sS0_=NULL;	
+        uS0_=nullptr;
+        sS0_=nullptr;
         ubulk_=new UnicodeString(src_,srcLen_);
-        sbulk_=NULL;
+        sbulk_=nullptr;
     }
 
-    StringPerfFunction(StdStringPerfFn func, UChar* source, int32_t sourceLen, UBool uselen)
+    StringPerfFunction(StdStringPerfFn func, char16_t* source, int32_t sourceLen, UBool uselen)
     {
 
         fn2_ = func;
-        lines_=NULL;
-        wlines_=NULL;
+        lines_=nullptr;
+        wlines_=nullptr;
         numLines_=0;
         uselen_=uselen;
         line_mode_=false;
-        src_ = new UChar[sourceLen];
+        src_ = new char16_t[sourceLen];
         memcpy(src_, source, sourceLen * U_SIZEOF_UCHAR);
         srcLen_ = sourceLen;
         fnType_ = Fn_STD;
 
-        uS0_=NULL;
-        sS0_=NULL;
-        ubulk_=NULL;
+        uS0_=nullptr;
+        sS0_=nullptr;
+        ubulk_=nullptr;
 
         //fillin wsrc_, sbulk_
         prepareBulkForStd();
@@ -241,7 +241,7 @@ public:
     }
 
 private:
-    void prepareLinesForStd(void)
+    void prepareLinesForStd()
     {
         UErrorCode err=U_ZERO_ERROR;
 
@@ -272,13 +272,13 @@ private:
 
     }
 
-    void prepareBulkForStd(void)
+    void prepareBulkForStd()
     {
         UErrorCode err=U_ZERO_ERROR;
 
-        const UChar* uSrc = src_;
+        const char16_t* uSrc = src_;
         int32_t uSrcLen = srcLen_;
-        wchar_t* wDest = NULL;
+        wchar_t* wDest = nullptr;
         int32_t wDestLen = 0;
         int32_t reqLen= 0 ;
 
@@ -288,7 +288,7 @@ private:
 
             if(err == U_BUFFER_OVERFLOW_ERROR){
                 err=U_ZERO_ERROR;
-                wDest =(wchar_t*) malloc(sizeof(wchar_t) * (reqLen));
+                wDest = static_cast<wchar_t*>(malloc(sizeof(wchar_t) * (reqLen)));
                 wDestLen = reqLen;
                 u_strToWCS(wDest,wDestLen,&reqLen,uSrc,uSrcLen,&err);
             }
@@ -305,7 +305,7 @@ private:
 
             if(err == U_BUFFER_OVERFLOW_ERROR){
                 err=U_ZERO_ERROR;
-                wDest =(wchar_t*) malloc(sizeof(wchar_t) * (reqLen+1));
+                wDest = static_cast<wchar_t*>(malloc(sizeof(wchar_t) * (reqLen + 1)));
                 wDestLen = reqLen+1;
                 u_strToWCS(wDest,wDestLen,&reqLen,uSrc,uSrcLen-1,&err);
             }
@@ -330,7 +330,7 @@ private:
     int32_t numLines_;
 
     UBool uselen_;
-    UChar* src_;
+    char16_t* src_;
     int32_t srcLen_;
     wchar_t* wsrc_;
     int32_t wsrcLen_;
@@ -350,9 +350,9 @@ class StringPerformanceTest : public UPerfTest
 public:
     StringPerformanceTest(int32_t argc, const char *argv[], UErrorCode &status);
     ~StringPerformanceTest();
-    virtual UPerfFunction* runIndexedTest(int32_t index, UBool exec,
-                                          const char *&name,
-                                          char *par = NULL);
+    UPerfFunction* runIndexedTest(int32_t index, UBool exec,
+                                  const char*& name,
+                                  char* par = nullptr) override;
     UPerfFunction* TestCtor();
     UPerfFunction* TestCtor1();
     UPerfFunction* TestCtor2();
@@ -382,60 +382,60 @@ public:
 private:
     long COUNT_;
     ULine* filelines_;
-    UChar* StrBuffer;
+    char16_t* StrBuffer;
     int32_t StrBufferLen;
 
 };
 
 
-inline void ctor(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void ctor(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     UnicodeString a;
 }
 
-inline void ctor1(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void ctor1(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     UnicodeString b(uTESTCHAR1);
 }
 
-inline void ctor2(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void ctor2(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     UnicodeString c(uEMPTY);
 }
 
-inline void ctor3(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void ctor3(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     UnicodeString d(src,srcLen);
 }
 
-inline UnicodeString icu_assign_helper(const UChar* src,int32_t srcLen)
+inline UnicodeString icu_assign_helper(const char16_t* src,int32_t srcLen)
 {
     if (srcLen==-1) { return src;}
     else { return UnicodeString(src, srcLen);}
 }
 
-inline void assign(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void assign(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     unistr = icu_assign_helper(src,srcLen);
 }
 
-inline void assign1(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void assign1(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     unistr.setTo(src, srcLen);
 }
 
-inline void assign2(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void assign2(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     unistr = s0;
 }
 
-inline void getch(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void getch(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     s0.charAt(0);
 }
 
 
-inline void catenate(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void catenate(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     UTimer mystart, mystop;
     utimer_getTime(&mystart);
@@ -443,7 +443,7 @@ inline void catenate(const UChar* src,int32_t srcLen, UnicodeString s0)
     *catICU += s0;
 
     utimer_getTime(&mystop);
-    double mytime = utimer_getDeltaSeconds(&mystart,&mystop);
+    utimer_getDeltaSeconds(&mystart,&mystop);
 
     *catICU += uCatenate_STR;
 }
@@ -451,21 +451,21 @@ inline void catenate(const UChar* src,int32_t srcLen, UnicodeString s0)
 volatile int scan_idx;
 U_STRING_DECL(SCAN1, "123", 3);
 
-inline void scan(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void scan(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
-    UChar c='.';
+    char16_t c='.';
     scan_idx = uScan_STRING.indexOf(c);
 }
 
-inline void scan1(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void scan1(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
     scan_idx = uScan_STRING.indexOf(SCAN1,3);
 }
 
-inline void scan2(const UChar* src,int32_t srcLen, UnicodeString s0)
+inline void scan2(const char16_t* src,int32_t srcLen, UnicodeString s0)
 {
-    UChar c1='s';
-    UChar c2='m';
+    char16_t c1='s';
+    char16_t c2='m';
     scan_idx = uScan_STRING.indexOf(c1);
     scan_idx = uScan_STRING.indexOf(c2);
 }
@@ -531,23 +531,23 @@ inline void StdLibCatenate(const wchar_t* src,int32_t srcLen, stlstring s0)
     *catStd += sCatenate_STR;
 
     utimer_getTime(&mystop);
-    double mytime = utimer_getDeltaSeconds(&mystart,&mystop);
+    utimer_getDeltaSeconds(&mystart,&mystop);
 
 }
 
 inline void StdLibScan(const wchar_t* src,int32_t srcLen, stlstring s0)
 {
-    scan_idx = (int) sScan_STRING.find('.');	
+    scan_idx = static_cast<int>(sScan_STRING.find('.'));
 }
 
 inline void StdLibScan1(const wchar_t* src,int32_t srcLen, stlstring s0)
 {
-    scan_idx = (int) sScan_STRING.find(L"123");
+    scan_idx = static_cast<int>(sScan_STRING.find(L"123"));
 }
 
 inline void StdLibScan2(const wchar_t* src,int32_t srcLen, stlstring s0)
 {	
-    scan_idx = (int) sScan_STRING.find_first_of(L"sm");
+    scan_idx = static_cast<int>(sScan_STRING.find_first_of(L"sm"));
 }
 
 #endif // STRINGPERF_H
